@@ -174,6 +174,8 @@ enum shader_target
 	TARGET_GS_COPY,
 	TARGET_COMPUTE,
 	TARGET_FETCH,
+	TARGET_HS,
+	TARGET_LS,
 
 	TARGET_NUM
 };
@@ -518,6 +520,8 @@ struct bc_alu {
 
 	unsigned slot:3;
 
+	unsigned lds_idx_offset:6;
+
 	alu_op_flags slot_flags;
 
 	void set_op(unsigned op) {
@@ -536,10 +540,12 @@ struct bc_fetch {
 
 	unsigned src_gpr:7;
 	unsigned src_rel:1;
+	unsigned src_rel_global:1; /* for GDS ops */
 	unsigned src_sel[4];
 
 	unsigned dst_gpr:7;
 	unsigned dst_rel:1;
+	unsigned dst_rel_global:1; /* for GDS ops */
 	unsigned dst_sel[4];
 
 	unsigned alt_const:1;
@@ -573,6 +579,7 @@ struct bc_fetch {
 	unsigned endian_swap:2;
 	unsigned mega_fetch:1;
 
+	unsigned src2_gpr:7; /* for GDS */
 	void set_op(unsigned op) { this->op = op; op_ptr = r600_isa_fetch(op); }
 };
 
@@ -652,6 +659,7 @@ public:
 			return false;
 
 		switch (hw_chip) {
+		case HW_CHIP_HEMLOCK:
 		case HW_CHIP_CYPRESS:
 		case HW_CHIP_JUNIPER:
 			return false;
@@ -739,6 +747,7 @@ private:
 	int decode_cf_mem(unsigned &i, bc_cf &bc);
 
 	int decode_fetch_vtx(unsigned &i, bc_fetch &bc);
+	int decode_fetch_gds(unsigned &i, bc_fetch &bc);
 };
 
 // bytecode format definition
@@ -779,13 +788,12 @@ public: \
 	} \
 	unsigned get_##name() const { \
 		return (value>>(first_bit))&((1ull<<((last_bit)-(first_bit)+1))-1); \
-	} \
+	}
 
 #define BC_RSRVD(fmt, last_bit, first_bit)
 
 // CLAMP macro defined elsewhere interferes with bytecode field name
 #undef CLAMP
-#undef ARRAY_SIZE
 #include "sb_bc_fmt_def.inc"
 
 #undef BC_FORMAT_BEGIN
